/* Copyright 2025 by COG9 Enterprises LLC
 * All rights reserved.
 * Title: powerard.h
 * Author: Jed Marti KI7NNP
 * Description: Control program and random prototypes.
 * Created: Sun Oct  5 18:58:03 2025
 * Revision History:
 *   Sun Dec 14 12:52:02 2025 - JBM add temp to storage.
 *   Sun Dec 28 10:15:39 2025 - JBM new ina219  
 *   Fri Feb  6 08:38:48 2026 - JBM start recording power on, HTML/SVG output of graphs.
 *   Sun Feb 22 18:20:07 2026 - JBM Remove INA219 calibration code, use direct computation.
 */

#ifndef powerard_h
#define powerard_h


#define VERSION 6
#define RELEASE 0
#define PATCH   3

// Size of input buffer.
#define SERIALBUFFER 80

// See batmon.ino
void calibrateBatMon(void);

// See sercmd.ino
void accumulateCharacter(void);
void serialCommand(void);

// See startup.ino - this is a screen roll.
void initRoll(void);
void rollDisplay(void);
void rollString(char *msg);

// Used to avoid println.
void terpri(void);

// See netmon.ino
void netpage(void);

// See fbat.ino
void batteryIcon(void);
uint8_t realBatteryCapacity(void);

// How often to update the screen (in milliseconds).
#define SCREENUPDATE 1000

#endif
